
var My_Messages  = new TinyMessage( "Note" );
var My_Series    = new E_Series( My_Messages );
var My_Deviation = new SimpleLabel( "LabelAbweichung" );

function set_deviation ( form, current, val ) {

   My_Deviation.reset();

   var real_current = My_Series.real( current, 'Ω', 'R' );
   var real_val     = My_Series.real( val, 'Ω', 'R' );
   if ( !real_val ) return;

   var deviation = 100 * ( real_current - real_val ) / real_val;
   if ( Math.abs(deviation) < 0.1 ) deviation = 0;
   My_Deviation.put( "Der Wert von R2 weicht um " + deviation.toFixed( 2 ) + "% ab." );
   return;

}; // set_deviation

function reset_all_messages () {
   My_Messages.reset();
   My_Deviation.reset();
}; // reset_all_messages

function do_B1_u( form, series ) {
   do_B1_clear ( form );
   reset_all_messages();
   var U  = My_Series.real( form.U.value,   'V', 'U' );
   var R2 = My_Series.real( form.R_2.value, 'Ω', 'R' );
   if ( My_Messages.error() ) return;

   if ( R2 ) {
      if ( R2 > 6862 ) {
         My_Messages.put_error( "Der Widerstand muss R2 ≤ 6,82kΩ sein." );
	 My_Messages.error();
	 return;
      };
      form.U.value = My_Series.stringify( 1.25+R2*0.00521, series.value, 'V' );
   } else {
      My_Messages.put_error( "Der Widerstand R2 ≤ 6,82kΩ muss angegeben werden." );
   }
   if ( My_Messages.error() ) return;
   My_Messages.put( "U=1,25V + R2 * 5,21mA" );
}; // do_B1_u

function do_B1_clear ( form ) {
   reset_all_messages();
   form.U.value = '';
}; // do_B1_clear

function do_B2_r2( form, series ) {
   do_B2_clear ( form );
   reset_all_messages();
   var U  = My_Series.real( form.U.value,   'V', 'U' );
   var R2 = My_Series.real( form.R_2.value, 'Ω', 'R' );
   if ( My_Messages.error() ) return;

   if ( U ) {
      if ( U < 1.25 ) {
         My_Messages.put_error( "Die Ausgangsspannung muss U ≥ 1,25V sein." );
	 My_Messages.error();
	 return;
      } else if ( U > 37 ) {
         My_Messages.put_error( "Die Ausgangsspannung muss U < 37V sein." );
	 My_Messages.error();
      };
      R2 = (U-1.25)/0.00521;
      form.R_2.value = My_Series.stringify( R2, series.value, 'Ω' );
      set_deviation( form, My_Series.stringify( R2, 'Exakt', 'Ω' ), form.R_2.value );
   } else {
      My_Messages.put_error( "Die Ausgangsspannung U ≥ 1,25V  muss angegeben werden." );
   }
   if ( My_Messages.error() ) return;
   My_Messages.put( "R2 = ( U - 1,25V ) / 5,21mA" );
}; // do_B2_r2

function do_B2_clear ( form ) {
   reset_all_messages();
   form.R2.value = '';
}; // do_B1_clear

function do_R1_exakt ( form, series ) {
   reset_all_messages();
   var U        = My_Series.real( form.U.value, 'V', 'U' );
   form.U.value = My_Series.stringify( U, 'Exakt', 'V' );
   return;
}; // do_R1_exakt

function do_R1_ing ( form, series ) {
   reset_all_messages();
   var U        = My_Series.real( form.U.value, 'V', 'U' );
   form.U.value = My_Series.stringify( U, 'Ing', 'V' );
}; // do_R1_ing

function do_R2_exakt ( form, series ) {
   reset_all_messages();
   var R2        = My_Series.real( form.R_2.value, 'Ω', 'R' );
   form.R_2.value = My_Series.stringify( R2, 'Exakt', 'Ω' );
}; // do_R2_exakt

function do_R2_ing ( form, series ) {
   reset_all_messages();
   var R2        = My_Series.real( form.R_2.value, 'Ω', 'R' );
   form.R_2.value = My_Series.stringify( R2, 'Ing', 'Ω' );
}; // do_R2_ing

function do_R2_e6 ( form, series ) {
   var current     = form.R_2.value;
   var R2          = My_Series.real( form.R_2.value, 'Ω', 'R' );
   form.R_2.value  = My_Series.stringify( R2, 'E6', 'Ω' );
   set_deviation( form, current, form.R_2.value );
}; // do_R2_e6

function do_R2_e12 ( form, series ) {
   var current    = form.R_2.value;
   var R2        = My_Series.real( form.R_2.value, 'Ω', 'R' );
   form.R_2.value = My_Series.stringify( R2, 'E12', 'Ω' );
   set_deviation( form, current, form.R_2.value );
}; // do_R2_e12

function do_R2_e24 ( form, series ) {
   var current   = form.R_2.value;
   var R2        = My_Series.real( form.R_2.value, 'Ω', 'R' );
   form.R_2.value = My_Series.stringify( R2, 'E24', 'Ω' );
   set_deviation( form, current, form.R_2.value );
}; // do_R2_e24

function do_R2_e96 ( form, series ) {
   var current   = form.R_2.value;
   var R2        = My_Series.real( form.R_2.value, 'Ω', 'R' );
   form.R_2.value = My_Series.stringify( R2, 'E96', 'Ω' );
   set_deviation( form, current, form.R_2.value );
}; // do_R2_e96

