
var My_Messages  = new TinyMessage( "Note" );
var My_Series    = new E_Series( My_Messages );
var My_Deviation = new SimpleLabel( "LabelAbweichung" );

var max_deviation = 0.001;

function set_deviation ( form, current, val ) {

   My_Deviation.reset();

   var real_current = My_Series.real( current, 'Ω', 'R' );
   var real_val     = My_Series.real( val, 'Ω', 'R' );
   if ( !real_val ) return;

   var deviation = 100 * ( real_current - real_val ) / real_val;
   if ( Math.abs(deviation) < 0.1 ) deviation = 0;
   My_Deviation.put( "Der berechnete Wert R weicht um " + deviation.toFixed( 2 ) + "% ab." );
   return;

}; // set_deviation

function reset_all_messages () {
   My_Messages.reset();
   My_Deviation.reset();
}; // reset_all_messages

function check_R_and_U( form ) {
   var U  = My_Series.real( form.U.value,   'V', 'U' );
   var U1 = My_Series.real( form.U1.value,  'V', 'U' );
   var U2 = My_Series.real( form.U2.value,  'V', 'U' );
   var R  = My_Series.real( form.R.value,   'Ω', 'R' );
   var R1 = My_Series.real( form.R_1.value, 'Ω', 'R' );
   var R2 = My_Series.real( form.R_2.value, 'Ω', 'R' );
   if ( My_Messages.error() ) return;

   var my_max_deviation = max_deviation;
   if ( form.R5.value == 'Exakt' || form.R5.value == 'Exakt' ) {
      my_max_deviation = 0.00000001;
   };
   if ( Math.abs( (R1+R2-R) / (R1+R2) ) > my_max_deviation ) {
      My_Messages.put( "Angepasst:   R = R1 + R2" );
      form.R.value   = My_Series.stringify( R1+R2, form.R4.value, 'Ω' );
   };

   my_max_deviation = max_deviation;
   if ( form.R2.value == 'Exakt' || form.R3.value == 'Exakt' ) {
      my_max_deviation = 0.00000001;
   };

   if ( Math.abs( 1 - ( ((U1/U2) / (R1/R2) ) ) ) > my_max_deviation ) {
      My_Messages.put( "Angepasst:   U1 = U * R1 / (R1+R2) und U2 = U * R2 / (R1+R2)" );
      form.U1.value  = My_Series.stringify( U*R1/(R1+R2), form.R2.value, 'V' );
      form.U2.value  = My_Series.stringify( U*R2/(R1+R2), form.R3.value, 'V' );
   }; 

   return;

}; // check_R_and_U

function do_B1_all( form, series ) {
   reset_all_messages();
   var U  = My_Series.real( form.U.value,   'V', 'U' );
   var U1 = My_Series.real( form.U1.value,  'V', 'U' );
   var U2 = My_Series.real( form.U2.value,  'V', 'U' );
   var R  = My_Series.real( form.R.value,   'Ω', 'R' );
   var R1 = My_Series.real( form.R_1.value, 'Ω', 'R' );
   var R2 = My_Series.real( form.R_2.value, 'Ω', 'R' );
   if ( My_Messages.error() ) return;

   var Message_U    = '';
   var Message_R    = '';
   var Selected     = '';
   var Number_of_Rs  = 0;
   var Number_of_Us = 0;
   if ( U  ) Number_of_Us++;
   if ( U1 ) Number_of_Us++;
   if ( U2 ) Number_of_Us++;
   if ( R  ) Number_of_Rs++;
   if ( R1 ) Number_of_Rs++;
   if ( R2 ) Number_of_Rs++;
   if ( Number_of_Us >= 3 ) {
      if ( Math.abs(U1+U2-U)/U > max_deviation ) {
         My_Messages.put_error( "Es sind die Werte U, U1 und U2 gegeben und U≠U1+U2" );
         My_Messages.put( "Am besten U löschen, dann wird U neu berechnet." );
         return;
      };
      Message_U = "U=U1+U2";
      Number_of_Us--;
      Selected = "U1, U2, ";
   } else {
      Selected = "U, U1, U2, ";
   };
   if ( Number_of_Rs >= 3 ) {
      if ( Math.abs(R1+R2-R)/R > max_deviation ) {
         My_Messages.put_error( "Es sind die Werte R, R1 und R2 gegeben und R≠R1+R2" );
         My_Messages.put( "Am besten R löschen, dann wird R neu berechnet." );
         return;
      };
      Message_R = "R=R1+R2";
      Number_of_Rs--;
      Selected = Selected + "R1 und R2";
   } else {
      Selected = Selected + "R, R1 und R2";
   };

   if ( !Number_of_Us && !Number_of_Rs ) {
      My_Messages.put_error( "Es werden drei Werte aus " + Selected + " benötigt. 1" );
   } else if ( !Number_of_Us ) {
      My_Messages.put_error( "Es wird ein Wert U>0, U1>0 oder U2>0 benötigt. 2" );
      return;
   } else if ( !Number_of_Rs ) {
      My_Messages.put_error( "Es wird ein Wert R>0, R1>0 oder R2>0 benötigt. 3" );
      return;
   } else if ( (Number_of_Us+Number_of_Rs) > 3 ) {
      My_Messages.put_error( "Es werden maximal drei Werte aus " + Selected + " benötigt. 4" );
      return;
   } else if ( (Number_of_Us+Number_of_Rs) < 3 ) {
      My_Messages.put_error( "Es werden drei Werte aus " + Selected + " benötigt. 5" );
      return;
   } else if ( U && U1 >= U ) {
      My_Messages.put_error( "Es muss U>U1 sein." );
      return;
   } else if ( U && U2 >= U ) {
      My_Messages.put_error( "Es muss U>U2 sein." );
      return;
   } else if ( R && R1 >= R ) {
      My_Messages.put_error( "Es muss R>R1 sein." );
      return;
   } else if ( R && R2 >= R ) {
      My_Messages.put_error( "Es muss R>R2 sein." );
      return;
   } else if ( U1 && U2 && R ) {
      if ( Message_U ) My_Messages.put( Message_U );
      form.R_1.value = My_Series.stringify( U1*R/(U1+U2),     form.R5.value, 'Ω' );
      form.R_2.value = My_Series.stringify( U2*R/(U1+U2),     form.R6.value, 'Ω' );
      form.U.value   = My_Series.stringify( U1+U2,            form.R1.value, 'V' );
   } else if ( U1 && U2 && R1 ) {
      if ( Message_U ) My_Messages.put( Message_U );
      form.R.value   = My_Series.stringify( (U1+U2)*R1/U1,    form.R4.value, 'Ω' );
      form.R_2.value = My_Series.stringify( U2*R1/U1,         form.R6.value, 'Ω' );
      form.U.value   = My_Series.stringify( U1+U2,            form.R1.value, 'V' );
   } else if ( U1 && U2 && R2 ) {
      if ( Message_U ) My_Messages.put( Message_U );
      form.R.value   = My_Series.stringify( (U1+U2)*R2/U2,    form.R4.value, 'Ω' );
      form.R_1.value = My_Series.stringify( U1*R2/U2,         form.R5.value, 'Ω' );
      form.U.value   = My_Series.stringify( U1+U2,            form.R1.value, 'V' );
   } else if ( U  && U1 && R  ) {
      form.R_1.value = My_Series.stringify( U1*R/U,           form.R5.value, 'Ω' );
      form.R_2.value = My_Series.stringify( (U-U1)*R/U,       form.R6.value, 'Ω' );
      form.U2.value  = My_Series.stringify( U-U1,             form.R3.value, 'V' );
   } else if ( U  && U2 && R  ) {
      form.R_1.value = My_Series.stringify( (U-U2)*R/U,       form.R5.value, 'Ω' );
      form.R_2.value = My_Series.stringify( U2*R/U,           form.R6.value, 'Ω' );
      form.U1.value  = My_Series.stringify( U-U2,             form.R2.value, 'V' );
   } else if ( U  && U1 && R1 ) {
      form.R.value   = My_Series.stringify( U*R1/U1 ,         form.R4.value, 'Ω' );
      form.R_2.value = My_Series.stringify( (U-U1)*R1/U1,     form.R6.value, 'Ω' );
      form.U2.value  = My_Series.stringify( U-U1,             form.R3.value, 'V' );
   } else if ( U  && U2 && R1 ) {
      form.R.value   = My_Series.stringify( U*R1/(U-U2) ,     form.R4.value, 'Ω' );
      form.R_2.value = My_Series.stringify( U2*R1/(U-U2),     form.R6.value, 'Ω' );
      form.U1.value  = My_Series.stringify( U-U2,             form.R2.value, 'V' );
   } else if ( U  && U1 && R2 ) {
      form.R.value   = My_Series.stringify( U*R2/(U-U1) ,     form.R4.value, 'Ω' );
      form.R_1.value = My_Series.stringify( (U-U1)*R2/(U-U1), form.R5.value, 'Ω' );
      form.U2.value  = My_Series.stringify( U-U1,             form.R3.value, 'V' );
   } else if ( U  && U2 && R2 ) {
      form.R.value   = My_Series.stringify( U*R2/U2 ,         form.R4.value, 'Ω' );
      form.R_1.value = My_Series.stringify( U2*R2/U2,         form.R5.value, 'Ω' );
      form.U1.value  = My_Series.stringify( U-U2,             form.R2.value, 'V' );
   } else if ( U  && R1 && R2 ) {
      if ( Message_R ) My_Messages.put( Message_R );
      form.R.value   = My_Series.stringify( R1+R2,            form.R4.value, 'Ω' );
      form.U1.value  = My_Series.stringify( R1*U/(R1+R2),     form.R2.value, 'V' );
      form.U2.value  = My_Series.stringify( R2*U/(R1+R2),     form.R3.value, 'V' );
   } else if ( U  && R  && R1 ) {
      form.R_2.value = My_Series.stringify( R-R1,             form.R6.value, 'Ω' );
      form.U1.value  = My_Series.stringify( R1*U/R,           form.R2.value, 'V' );
      form.U2.value  = My_Series.stringify( U-R1*U/R,         form.R3.value, 'V' );
   } else if ( U  && R  && R2 ) {
      form.R_1.value = My_Series.stringify( R-R2,             form.R5.value, 'Ω' );
      form.U1.value  = My_Series.stringify( U-R2*U/R,         form.R2.value, 'V' );
      form.U2.value  = My_Series.stringify( R2*U/R,           form.R3.value, 'V' );
   } else if ( U1 && R1 && R2 ) {
      if ( Message_R ) My_Messages.put( Message_R );
      form.R.value   = My_Series.stringify( R1+R2,              form.R4.value, 'Ω' );
      form.U.value   = My_Series.stringify( (R1+R2)*U1/R1,     form.R1.value, 'V' );
      form.U2.value  = My_Series.stringify( R2*U1/R1,         form.R3.value, 'V' );
   } else if ( U1 && R  && R1 ) {
      form.R_2.value = My_Series.stringify( R-R1,             form.R6.value, 'Ω' );
      form.U1.value  = My_Series.stringify( R1*U/R,           form.R2.value, 'V' );
      form.U2.value  = My_Series.stringify( (R-R1)*U/R,       form.R3.value, 'V' );
   } else if ( U1 && R  && R2 ) {
      form.R_1.value = My_Series.stringify( R-R2,             form.R5.value, 'Ω' );
      form.U1.value  = My_Series.stringify( (R-R2)*U/R,       form.R2.value, 'V' );
      form.U2.value  = My_Series.stringify( R2*U/R,           form.R3.value, 'V' );
   } else if ( U2 && R1 && R2 ) {
      if ( Message_R ) My_Messages.put( Message_R );
      form.R.value   = My_Series.stringify( R1+R2,              form.R4.value, 'Ω' );
      form.U.value   = My_Series.stringify( (R1+R2)*U2/R2,     form.R1.value, 'V' );
      form.U1.value  = My_Series.stringify( R1*U2/R2,         form.R2.value, 'V' );
   } else if ( U2 && R  && R1 ) {
      form.R_2.value = My_Series.stringify( R-R1,             form.R6.value, 'Ω' );
      form.U.value   = My_Series.stringify( R*U2/(R-R1),       form.R1.value, 'V' );
      form.U1.value  = My_Series.stringify( R1*U2/(R-R1),     form.R2.value, 'V' );
   } else if ( U2 && R  && R2 ) {
      form.R_1.value = My_Series.stringify( R-R2,             form.R5.value, 'Ω' );
      form.U.value   = My_Series.stringify( R*U2/R2,           form.R1.value, 'V' );
      form.U1.value  = My_Series.stringify( R1*U2/R2,         form.R2.value, 'V' );
   } else {
      My_Messages.put_error( "Fehler im Script :-(" );
      return;
   };

   check_R_and_U( form ); 

   U = My_Series.real( form.U.value,  'V', 'U' );
   R = My_Series.real( form.R.value,  'Ω', 'R' );
   form.I.value = My_Series.stringify( U/R, form.R7.value, 'A' );

   return;

} // do_B1_all

function do_B1_u( form, series ) {
   do_B1_clear ( form );
   do_B1_all( form, series );
   if ( My_Messages.error() ) return;
   My_Messages.put( "U = U1+U2" );
}; // do_B1_u

function do_B1_clear ( form ) {
   reset_all_messages();
   form.U.value = '';
}; // do_B1_clear

function do_B2_u1( form, series ) {
   do_B2_clear ( form );
   do_B1_all( form, series );
   if ( My_Messages.error() ) return;
   My_Messages.put( "U1 = R1 * U2/R2" );
}; // do_B2_u1

function do_B2_clear ( form ) {
   reset_all_messages();
   form.U1.value = '';
}; // do_B1_clear

function do_B3_u2( form, series ) {
   do_B3_clear ( form );
   do_B1_all( form, series );
   if ( My_Messages.error() ) return;
   My_Messages.put( "U2 = R2 * U1/R1" );
}; // do_B3_u2

function do_B3_clear ( form ) {
   reset_all_messages();
   form.U2.value = '';
}; // do_B3_clear

function do_B4_r( form, series ) {
   do_B4_clear ( form );
   do_B1_all( form, series );
   if ( My_Messages.error() ) return;
   My_Messages.put( "R = R1+R2" );
}; // do_B4_r

function do_B4_clear ( form ) {
   reset_all_messages();
   form.R.value = '';
}; // do_B4_clear

function do_B5_r1( form, series ) {
   do_B5_clear ( form );
   do_B1_all( form, series );
   if ( My_Messages.error() ) return;
   My_Messages.put( "R1 = U1 * R2/U2" );
}; // do_B5_r1

function do_B5_clear ( form ) {
   reset_all_messages();
   form.R_1.value = '';
}; // do_B5_clear

function do_B6_r2( form, series ) {
   do_B6_clear ( form );
   do_B1_all( form, series );
   if ( My_Messages.error() ) return;
   My_Messages.put( "R2 = U2 * R1/U1" );
}; // do_B6_r2

function do_B6_clear ( form ) {
   reset_all_messages();
   form.R_2.value = '';
}; // do_B6_clear

function do_B7_i( form, series ) {
   reset_all_messages();
   var U1 = My_Series.real( form.U1.value, 'V', 'U' );
   var U2 = My_Series.real( form.U2.value, 'V', 'U' );
   var R1 = My_Series.real( form.R_1.value, 'Ω', 'R' );
   var R2 = My_Series.real( form.R_2.value, 'Ω', 'R' );
   if ( My_Messages.error() ) return;

   if ( !U1 || !U2 || !R1 || !R2 ) {
      do_B1_all( form, series );
   } else {
      form.I.value = My_Series.stringify( (U1+U2)/(R1+R2), form.R7.value, 'A' );
   };
   if ( My_Messages.error() ) return;
   My_Messages.put( "I=(U1+U2)/(R1+R2)" );
   return;

}; // do_B7_i

function do_B7_clear ( form ) {
   reset_all_messages();
   form.I.value = '';
}; // do_B7_clear

function do_R1_exakt ( form, series ) {
   reset_all_messages();
   var U        = My_Series.real( form.U.value, 'V', 'U' );
   form.U.value = My_Series.stringify( U, 'Exakt', 'V' );
   return;
}; // do_R1_exakt

function do_R1_ing ( form, series ) {
   reset_all_messages();
   var U        = My_Series.real( form.U.value, 'V', 'U' );
   form.U.value = My_Series.stringify( U, 'Ing', 'V' );
}; // do_R1_ing

function do_R2_exakt ( form, series ) {
   reset_all_messages();
   var U1        = My_Series.real( form.U1.value, 'V', 'U' );
   form.U1.value = My_Series.stringify( U1, 'Exakt', 'V' );
}; // do_R2_exakt

function do_R2_ing ( form, series ) {
   reset_all_messages();
   var U1        = My_Series.real( form.U1.value, 'V', 'U' );
   form.U1.value = My_Series.stringify( U1, 'Ing', 'V' );
}; // do_R2_ing

function do_R3_exakt ( form, series ) {
   reset_all_messages();
   var U2        = My_Series.real( form.U2.value, 'V', 'U' );
   form.U2.value = My_Series.stringify( U2, 'Exakt', 'V' );
}; // do_R3_exakt

function do_R3_ing ( form, series ) {
   reset_all_messages();
   var U2        = My_Series.real( form.U2.value, 'V', 'U' );
   form.U2.value = My_Series.stringify( U2, 'Ing', 'V' );
}; // do_R3_ing

function do_R4_exakt ( form, series ) {
   reset_all_messages();
   var R        = My_Series.real( form.R.value, 'Ω', 'R' );
   form.R.value = My_Series.stringify( R, 'Exakt', 'Ω' );
}; // do_R4_exakt

function do_R4_ing ( form, series ) {
   reset_all_messages();
   var R        = My_Series.real( form.R.value, 'Ω', 'R' );
   form.R.value = My_Series.stringify( R, 'Ing', 'Ω' );
}; // do_R4_ing

function do_R5_exakt ( form, series ) {
   reset_all_messages();
   var R1         = My_Series.real( form.R_1.value, 'Ω', 'R' );
   form.R_1.value = My_Series.stringify( R1, 'Exakt', 'Ω' );
}; // do_R5_exakt

function do_R5_ing ( form, series ) {
   reset_all_messages();
   var R1         = My_Series.real( form.R_1.value, 'Ω', 'R' );
   form.R_1.value = My_Series.stringify( R1, 'Ing', 'Ω' );
}; // do_R5_ing

function do_R5_e6 ( form, series ) {
   var current    = form.R_1.value;
   var R1         = My_Series.real( form.R_1.value, 'Ω', 'R' );
   form.R_1.value = My_Series.stringify( R1, 'E6', 'Ω' );
   set_deviation( form, current, form.R_1.value );
}; // do_R5_e6

function do_R5_e12 ( form, series ) {
   var current    = form.R_1.value;
   var R1         = My_Series.real( form.R_1.value, 'Ω', 'R' );
   form.R_1.value = My_Series.stringify( R1, 'E12', 'Ω' );
   set_deviation( form, current, form.R_1.value );
}; // do_R5_e12

function do_R5_e24 ( form, series ) {
   var current    = form.R_1.value;
   var R1         = My_Series.real( form.R_1.value, 'Ω', 'R' );
   form.R_1.value = My_Series.stringify( R1, 'E24', 'Ω' );
   set_deviation( form, current, form.R_1.value );
}; // do_R5_e24

function do_R5_e96 ( form, series ) {
   var current    = form.R_1.value;
   var R1         = My_Series.real( form.R_1.value, 'Ω', 'R' );
   form.R_1.value = My_Series.stringify( R1, 'E96', 'Ω' );
   set_deviation( form, current, form.R_1.value );
}; // do_R5_e96

function do_R6_exakt ( form, series ) {
   reset_all_messages();
   var R2         = My_Series.real( form.R_2.value, 'Ω', 'R' );
   form.R_2.value = My_Series.stringify( R2, 'Exakt', 'Ω' );
}; // do_R6_exakt

function do_R6_ing ( form, series ) {
   reset_all_messages();
   var R2         = My_Series.real( form.R_2.value, 'Ω', 'R' );
   form.R_2.value = My_Series.stringify( R2, 'Ing', 'Ω' );
}; // do_R6_ing

function do_R6_e6 ( form, series ) {
   var current    = form.R_2.value;
   var R2         = My_Series.real( form.R_2.value, 'Ω', 'R' );
   form.R_2.value = My_Series.stringify( R2, 'E6', 'Ω' );
   set_deviation( form, current, form.R_2.value );
}; // do_R6_e6

function do_R6_e12 ( form, series ) {
   var current    = form.R_2.value;
   var R2         = My_Series.real( form.R_2.value, 'Ω', 'R' );
   form.R_2.value = My_Series.stringify( R2, 'E12', 'Ω' );
   set_deviation( form, current, form.R_2.value );
}; // do_R6_e12

function do_R6_e24 ( form, series ) {
   var current    = form.R_2.value;
   var R2         = My_Series.real( form.R_2.value, 'Ω', 'R' );
   form.R_2.value = My_Series.stringify( R2, 'E24', 'Ω' );
   set_deviation( form, current, form.R_2.value );
}; // do_R6_e24

function do_R6_e96 ( form, series ) {
   var current    = form.R_2.value;
   var R2         = My_Series.real( form.R_2.value, 'Ω', 'R' );
   form.R_2.value = My_Series.stringify( R2, 'E96', 'Ω' );
   set_deviation( form, current, form.R_2.value );
}; // do_R6_e96

function do_R7_exakt ( form, series ) {
   reset_all_messages();
   var I        = My_Series.real( form.I.value, 'A', 'I' );
   form.I.value = My_Series.stringify( I, 'Exakt', 'A' );
   return;
}; // do_R7_exakt

function do_R7_ing ( form, series ) {
   reset_all_messages();
   var I        = My_Series.real( form.I.value, 'A', 'I' );
   form.I.value = My_Series.stringify( I, 'Ing', 'A' );
}; // do_R7_ing

