
var My_Messages       = new TinyMessage( "Note" );
var My_Series         = new E_Series( My_Messages );
var My_Deviation      = new SimpleLabel( "LabelAbweichung" );

var C_max_frequency = 500e3;
var C_min_C         = 100e-12;
var C_max_C         = 10e-3;
var C_min_R         = 1e3;
var C_max_R         = 3e6;
var C_ln_2          = 0.693147180559945; // Math.log( 2 );
var C_1_ln_2        = 1.44269504088896; // 1 / Math.log( 2 );
var Ct_ln_2         = 0.693; // Math.log( 2 );
var Ct_1_ln_2       = 1.44; // 1 / Math.log( 2 );

function Astabil555_init_me () {
   My_Deviation.reset();
   My_Messages.reset();
   document.form1.Rt1.value = '';
   document.form1.Rt2.value = '';
   document.form1.C.value   = '';
   return;
}; // Astabil555_init_me

function calculate_period_and_frequency ( form ) {

   var f        = My_Series.real( form.f.value,  'Hz', 'f' );
   var t        = My_Series.real( form.t.value,  's',  't' );
   var th       = My_Series.real( form.th.value, 's',  't' );
   var tl       = My_Series.real( form.tl.value, 's',  't' );
   var tg       = My_Series.real( form.tg.value, '',   ''  );
   var seriesf  = form.R1.value;
   var seriest  = form.R2.value;
   var seriesth = form.R6.value;
   var seriestl = form.R7.value;

   if ( !t ) {
      if ( th && tl ) {
         if ( th <= tl ) {
            My_Messages.put_error( "th=" + My_Series.stringify( th, seriesth, 's' ) + " <= tl=" +
                                   My_Series.stringify( tl, seriestl, 's' ) + " ist nicht möglich." );
            return ''
         };
         t = th + tl;
         form.t.value  = My_Series.stringify( t, seriest, 's' );
      } else if ( !th && !tl && !tg ) {
         // nop
      } else if ( tg && (tg >= 1) ) {
          My_Messages.put_error( "tg=" + tg + " >= 1 ist nicht erlaubt." );
          return '';
      } else if ( tg && (tg <= 0.5) ) {
          My_Messages.put_error( "tg=" + tg + " <= 0.5 ist nicht erlaubt." );
          return '';
      } else if ( th && tg ) {
         t = th / tg;
         form.t.value  = My_Series.stringify( t, seriest, 's' );
         form.tl.value = My_Series.stringify( t-th, seriestl, 's' );
      } else if ( tl && tg ) {
         t = tl / (1-tg);
         form.t.value  = My_Series.stringify( t, seriest, 's' );
         form.th.value = My_Series.stringify( t-tl, seriesth, 's' );
      };
   } else {
      check_times ( form, t, th, tl, tg, seriesth, seriestl );
   };
   if ( !t && !f ) return '';
   if ( f ) {
      if ( f > C_max_frequency ) {
         My_Messages.put_error( "f=" + My_Series.stringify( f, 'Ing', 'Hz' ) + ">" +
                                       My_Series.stringify( C_max_frequency, 'Ing', 'Hz' ) + " ist zu groß."
                              );
         return '';
      };
      t = 1 / f;
      form.t.value = My_Series.stringify( t, seriest, 's' );
      check_times ( form, t, th, tl, tg, seriesth, seriestl );
      return f;
   } else if ( 1/t > C_max_frequency ) {
      My_Messages.put_error( "t=" + My_Series.stringify( t, 'Ing', 's' ) + "<" +
                                    My_Series.stringify( 1/C_max_frequency, 'Ing', 's' ) + " ist zu kein."
                           );
      return '';
   } else {
      form.f.value = My_Series.stringify( 1/t, seriesf, 'Hz' );
      return 1 / t;
   };
}; // calculate_period_and_frequency

function check_times ( form, t, th, tl, tg, seriesth, seriestl ) {

   if ( th && (th >= t) ) {
      My_Messages.put_error( "th=" + th + " >= t=" + t + " ist nicht möglich." );
   } else if ( tl && (tl >= t) ) {
      My_Messages.put_error( "tl=" + tl + " >= t=" + t + " ist nicht möglich." );
   } else if ( th && tl ) {
      if ( (th+tl-t)/t > 0.01 ) {
         My_Messages.put_error( "th=" + th + ", tl=" + tl + ", th+tl=" + th+tl + " > t=" + t + " ist nicht erlaubt." );
      };
   } else if ( th ) {
      tl = t - th;
      form.tl.value = My_Series.stringify( t-th, seriestl, 's' );
   } else if ( tl ) {
      th = t - tl;
      form.tl.value = My_Series.stringify( t-th, seriestl, 's' );
   } else if ( tg && (tg >= 1) ) {
      My_Messages.put_error( "tg=" + tg + " >= 1 ist nicht erlaubt." );
   } else if ( tg && (tg <= 0.5) ) {
      My_Messages.put_error( "tg=" + tg + " <= 0.5 ist nicht erlaubt." );
   } else if ( tg ) {
      th = tg * t;
      tl = t - th;
      form.th.value = My_Series.stringify( th, seriesth, 's' );
      form.tl.value = My_Series.stringify( t-th, seriestl, 's' );
   } else {
      form.th.value = '';
      form.tl.value = '';
      form.tg.value = '';
   };

   return;

}; // check_times

function calculate_resistors_and_capacitor ( f, form, info ) {

   if ( !f )                  return [ '', '', '' ];
   if ( f < 0 )               return [ '', '', '' ];
   if ( f > C_max_frequency ) return [ '', '', '' ];

   var C       = C_min_C;
   var min_R   = 3 * C_min_R;
   var max_R   = 3 * C_max_R;
   // first try
   var R       = C_1_ln_2 / ( f  * C );
   var limit  = 0;
   while ( ! ( (R >= min_R) && (R <= max_R) ) ) {
      C = C * 10;
      R = C_1_ln_2 / ( f  * C );
      if ( limit > 10000 ) return [ '', '', '' ];
      limit++;
   };

   return calculate_resistors ( f, C, form, info );

}; // calculate_resistors_and_capacitor

function calculate_resistors ( f, C, form, info ) {

   if ( !f )                  return [ '', '', '' ];
   if ( f < 0 )               return [ '', '', '' ];
   if ( f > C_max_frequency ) return [ '', '', '' ];

   if ( !C )                  return [ '', '', '' ];
   if ( C < C_min_C )         return [ '', '', '' ];

   var th      = My_Series.real( form.th.value, 's', 't' );
   var tl      = My_Series.real( form.tl.value, 's', 't' );
   var series1 = form.R3.value;
   var series2 = form.R4.value;

   if ( th && tl ) {
      var Rt2 = tl / ( C * C_ln_2 );
      var Rt1 = ( th / ( C * C_ln_2 ) ) -  Rt2;
      if ( Rt1 < C_min_R ) {
         var tg = th / (th+tl);
            My_Messages.put_error( "tg=" + tg + " ist fast 0.5 und R1=" +
                                   My_Series.stringify( Rt1, 'Ing', 'Ω' ) + " ist zu klein bei R2=" + My_Series.stringify( Rt2, 'Ing', 'Ω' )
                                 );
            return [ '', '', '' ];
      };
      if ( info ) {
         My_Messages.put( "R2 = tl / ( C * " + Ct_ln_2 + " )" );
         My_Messages.put( "R1 = th / ( C * " + Ct_ln_2 + " ) - R2" );
      };
      return [ Rt1, Rt2, C ];
   };

   var min_R   = 3 * C_min_R;
   var max_R   = 3 * C_max_R;

   var R       = C_1_ln_2 / ( f  * C );
   if ( ! ( (R >= min_R) && (R <= max_R) ) ) return [ '', '', '' ];

   // choose best value for Rt1
   // start with a lower value ----------------v             
   var Rt1     = My_Series.E_value( (R / 3) / My_Series.E_multiplyer(series1), series1 );
   var Rt2     = My_Series.E_value( (R / 3), series2 );
   var f_value = C_1_ln_2 / ( (Rt1 + 2 * Rt2) * C );
   var delta   = (( f - f_value ) / f) ** 2;
   var optimal = 1.01 ** 2;  // 1%
   var Rt1_opt = Rt1;

   // if the first try for Rt1 fails try with a lower or higher value of Rt2
   Choose_R2: for ( i=0; 1<2; i++ ) {

      Choose_R1: while ( ( (Rt1 >= C_min_R) && (Rt1 <= C_max_R) && (Rt2 >= C_min_R) && (Rt2 <= C_max_R) ) ) {
         Rt1           = My_Series.E_value( (C_1_ln_2 / ( f * C )) - 2 * Rt2, series1 );
         f_value       = C_1_ln_2 / ( (Rt1 + 2 * Rt2) * C );
         var new_delta = (( f - f_value ) / f) ** 2;
         if ( new_delta < optimal ) {
            Rt1_opt = Rt1;
            break Choose_R1;
         };
         if ( new_delta < delta ) {
            delta   = new_delta;
            Rt1_opt = Rt1;
         };
         Rt1 = Rt1 * My_Series.E_multiplyer( series1 );
      }; // Coose_R1

      if ( Rt1 > C_max_R ) {
         Rt2 = My_Series.E_value( Rt2 / My_Series.E_multiplyer( series2 ), series2 );
      } else if ( Rt1 < C_min_R ) {
         Rt2 = My_Series.E_value( Rt2 * My_Series.E_multiplyer( series2 ), series2 );
      } else {
         break Choose_R2;
      };
   }; // Choose_R2

   if ( info ) My_Messages.put( "Wählte R1 und R2, so dass f = " + Ct_1_ln_2 + " / ( (Rt1 + 2 * Rt2) * C )" );

   return [ Rt1_opt, Rt2, C ];

}; // calculate_resistors

function suggest_RC ( form ) {
   var f   = calculate_period_and_frequency( form );
   var RRC = calculate_resistors_and_capacitor( f, form, false );
   var R1  = RRC[0];
   var R2  = RRC[1];
   var C   = RRC[2];
   if ( !f || !R1 || !R2 || !C ) return;
   My_Messages.put( "C="         + My_Series.stringify( C, 'Ing', 'F' ) +
                    " und R1="   + My_Series.stringify( R1, 'Ing', 'Ω' ) +
                    " sowie R2=" + My_Series.stringify( R2, 'Ing', 'Ω' ) + " wären möglich."
                  );
   return;
}; // suggest_RC

function set_deviation ( form ) {
   var f   = calculate_period_and_frequency( form );
   var t   = My_Series.real( form.t.value, 's', 't' );
   var Rt1 = My_Series.real( form.Rt1.value, 'Ω', 'R1' );
   var Rt2 = My_Series.real( form.Rt2.value, 'Ω', 'R2' );
   var C   = My_Series.real( form.C.value, 'F', 'C' );
   if ( My_Messages.error() || !f || !Rt1 || !Rt2 || !C ) {
      My_Deviation.reset();
      return;
   };

   if ( !form.th.value ) My_Messages.put( "th = " + Ct_ln_2 + " * ( R1 + R2 ) * C" );
   if ( !form.tl.value ) My_Messages.put( "tl = " + Ct_ln_2 + " * R2 * C" );
   if ( !form.tg.value ) My_Messages.put( "tg = th / ( th + tl ) = ( R1 + R2 ) / ( R1 + 2 * R2 )" );

   var seriesth = form.R6.value;
   var seriestl = form.R7.value;
   var th       = C_ln_2 * ( Rt1 + Rt2 ) * C;
   var tl       = t - th;
   var tg       = th / t;
   form.th.value = My_Series.stringify( th, seriesth, 's' );
   form.tl.value = My_Series.stringify( tl, seriestl, 's' );
   form.tg.value = My_Series.stringify( tg, 'Exakt', '' );
 
   var f_value   = C_1_ln_2 / ( (Rt1 + 2 * Rt2) * C );
   var deviation = 100 * ( f - f_value ) / f;
   if ( Math.abs(deviation) < 0.1 ) deviation = 0;
   My_Deviation.put( "Der berechnete Wert f weicht um " + deviation.toFixed( 2 ) + "% ab." );
   return;
}; // set_deviation

// Do calculations

function do_B1_frequenz( form, series ) {
   My_Messages.reset();
   var Rt1 = My_Series.real( form.Rt1.value, 'Ω', 'R1' );
   var Rt2 = My_Series.real( form.Rt2.value, 'Ω', 'R2' );
   var C   = My_Series.real( form.C.value, 'F', 'C' );
   if ( My_Messages.error() ) return;
   if ( (!Rt1 || !Rt2 || !C) ) {
      My_Messages.put_error( "Es werden R1>0, R2>0 und C>0 benötigt." );
      suggest_RC( form );
   } else if ( Rt1> C_max_R ) {
      My_Messages.put_error( "R1 = " + My_Series.stringify( Rt1, 'Ing', 'Ω' ) + " > " + My_Series.stringify( C_max_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else if ( Rt2> C_max_R ) {
      My_Messages.put_error( "R2 = " + My_Series.stringify( Rt2, 'Ing', 'Ω' ) + " > " + My_Series.stringify( C_max_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else if ( Rt1 < C_min_R ) {
      My_Messages.put_error( "R1 = " + My_Series.stringify( Rt1, 'Ing', 'Ω' ) + " < " + My_Series.stringify( C_min_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else if ( Rt2 < C_min_R ) {
      My_Messages.put_error( "R2 = " + My_Series.stringify( Rt2, 'Ing', 'Ω' ) + " < " + My_Series.stringify( C_min_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else if ( C < C_min_C ) {
      My_Messages.put_error( "C = " + My_Series.stringify( C, 'Ing', 'F' ) + " < " + My_Series.stringify( C_min_C, 'Ing', 'F' ) );
      suggest_RC( form );
   } else {
      form.f.value = My_Series.stringify( C_1_ln_2 / ( (Rt1 + 2 * Rt2) * C ), series.value, 'Hz' );
      form.t.value = My_Series.stringify( ( (Rt1 + 2 * Rt2) * C ) / C_1_ln_2, series.value, 's' );
      set_deviation( form );
      My_Messages.put( "f = " + Ct_1_ln_2 + " / ( (R1 + 2 * R2) * C )" );
      My_Messages.put( "t = 1 / f = ( (R1 + 2 * R2) * C ) / " + Ct_1_ln_2 );
   };
   return;
} // do_B1_frequenz

function do_B2_periode( form, series ) {
   do_B1_frequenz( form, series );
   return;
} // do_B2_periode

function do_B3_optimize( form ) {
   My_Messages.reset();
   var f   = calculate_period_and_frequency( form );
   var C   = My_Series.real( form.C.value, 'F', 'C' );
   if ( My_Messages.error() ) return;
   if ( ( !C || !f ) ) {
      My_Messages.put_error( "Es werden C und f benötigt." );
      suggest_RC( form );
   } else if ( C == '' ) {
      My_Messages.put_error( "Es wird C>"+ My_Series.stringify( C_min_C, 'Ing', 'F' ) + " benötigt." );
      suggest_RC( form );
   } else if ( C < C_min_C ) {
      My_Messages.put_error( "C = " + My_Series.stringify( C, 'Ing', 'F' ) + " < " + My_Series.stringify( C_min_C, 'Ing', 'F' ) );
      suggest_RC( form );
   } else {
      var RRC = calculate_resistors( f, C, form, true );
      var R1  = RRC[0];
      var R2  = RRC[1];
      if ( !R1 || !R2 || !C ) {
         My_Messages.put_error( "Für f = " + My_Series.stringify( f, 'Ing', 'Hz' ) + " und C = " + My_Series.stringify( C, 'Ing', 'F' )
                                + " konnten R1 und R2 nicht ermittelt werden." );
         return
      };
      var series1 = form.R3.value;
      form.Rt1.value = My_Series.stringify( R1, series1, 'Ω' );
      var series2 = form.R4.value;
      form.Rt2.value = My_Series.stringify( R2, series2, 'Ω' );
      set_deviation( form );
   };
   return;
} // do_B3_optimize

function do_B3_ohm( form, series ) {
   My_Messages.reset();
   var f   = calculate_period_and_frequency( form );
   var Rt2 = My_Series.real( form.Rt2.value, 'Ω', 'R2' );
   var C   = My_Series.real( form.C.value, 'F', 'C' );
   if ( My_Messages.error() ) return;
   if ( (!Rt2 || !C || !f ) ) {
      My_Messages.put_error( "Es werden R2, C und f benötigt." );
      suggest_RC( form );
   } else if ( C == '' ) {
      My_Messages.put_error( "Es wird C>"+ My_Series.stringify( C_min_C, 'Ing', 'F' ) + " benötigt." );
      suggest_RC( form );
   } else if ( C < C_min_C ) {
      My_Messages.put_error( "C = " + My_Series.stringify( C, 'Ing', 'F' ) + " < " + My_Series.stringify( C_min_C, 'Ing', 'F' ) );
      suggest_RC( form );
   } else {
      var Rt1 = (C_1_ln_2 / ( f * C )) - 2 * Rt2;
      if ( Rt1 > C_max_R ) {
         My_Messages.put_error( "R1 = " + My_Series.stringify( Rt1, 'Ing', 'Ω' ) + " > " + My_Series.stringify( C_max_R, 'Ing', 'Ω' ) );
         suggest_RC( form );
      } else if ( Rt1 < C_min_R ) {
         My_Messages.put_error( "R1 = " + My_Series.stringify( Rt1, 'Ing', 'Ω' ) + " < " + My_Series.stringify( C_min_R, 'Ing', 'Ω' ) );
         suggest_RC( form );
      } else {
         form.Rt1.value = My_Series.stringify( Rt1, series.value, 'Ω' );
         set_deviation( form );
         My_Messages.put( "R1 = ( " + Ct_1_ln_2 + " / (f * C) ) - (2* R2)" );
      };
   };
   return;
} // do_B3_ohm

function do_B4_ohm( form, series ) {
   My_Messages.reset();
   var f   = calculate_period_and_frequency( form );
   var Rt1 = My_Series.real( form.Rt1.value, 'Ω', 'R1' );
   var C   = My_Series.real( form.C.value, 'F', 'C' );
   if ( My_Messages.error() ) return;
   if ( (!Rt1 || !C || !f ) ) {
      My_Messages.put_error( "Es werden R1, C und f benötigt." );
      suggest_RC( form );
   } else if ( C == '' ) {
      My_Messages.put_error( "Es wird C>"+ My_Series.stringify( C_min_C, 'Ing', 'F' ) + " benötigt." );
      suggest_RC( form );
   } else if ( C < C_min_C ) {
      My_Messages.put_error( "C = " + My_Series.stringify( C, 'Ing', 'F' ) + " < " + My_Series.stringify( C_min_C, 'Ing', 'F' ) );
      suggest_RC( form );
   } else {
      var Rt2 = ((C_1_ln_2 / ( f * C )) - Rt1) / 2;
      if ( Rt2 > C_max_R ) {
         My_Messages.put_error( "R2 = " + My_Series.stringify( Rt2, 'Ing', 'Ω' ) + " > " + My_Series.stringify( C_max_R, 'Ing', 'Ω' ) );
         suggest_RC( form );
      } else if ( Rt2 < C_min_R ) {
         My_Messages.put_error( "R2 = " + My_Series.stringify( Rt2, 'Ing', 'Ω' ) + " < " + My_Series.stringify( C_min_R, 'Ing', 'Ω' ) );
         suggest_RC( form );
      } else {
         form.Rt2.value = My_Series.stringify( Rt2, series.value, 'Ω' );
         set_deviation( form );
         My_Messages.put( "R2 = ( ( " + Ct_1_ln_2 + " / (f * C) ) - R1 ) / 2" );
      };
   };
   return;
} // do_B4_ohm

function do_B5_cond( form, series ) {
   My_Messages.reset();
   var f   = calculate_period_and_frequency( form );
   var Rt1 = My_Series.real( form.Rt1.value, 'Ω', 'R1' );
   var Rt2 = My_Series.real( form.Rt2.value, 'Ω', 'R2' );
   if ( My_Messages.error() ) return;
   if ( (!Rt1 || !Rt2 || !f ) ) {
      My_Messages.put_error( "Es werden R1, R2 und f benötigt." );
      suggest_RC( form );
   } else if ( Rt1 > C_max_R ) {
      My_Messages.put_error( "R1 = " + My_Series.stringify( Rt1, 'Ing', 'Ω' ) + " > " + My_Series.stringify( C_max_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else if ( Rt1 < C_min_R ) {
      My_Messages.put_error( "R1 = " + My_Series.stringify( Rt1, 'Ing', 'Ω' ) + " < " + My_Series.stringify( C_min_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else if ( Rt2 > C_max_R ) {
      My_Messages.put_error( "R2 = " + My_Series.stringify( Rt2, 'Ing', 'Ω' ) + " > " + My_Series.stringify( C_max_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else if ( Rt2 < C_min_R ) {
      My_Messages.put_error( "R2 = " + My_Series.stringify( Rt2, 'Ing', 'Ω' ) + " < " + My_Series.stringify( C_min_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else {
      var C = C_1_ln_2 / ( f * (Rt1 + 2 * Rt2) );
      if ( C < C_min_C ) {
         My_Messages.put_error( "C = " + My_Series.stringify( C, 'Ing', 'F' ) + " < " + My_Series.stringify( C_min_C, 'Ing', 'F' ) );
         suggest_RC( form );
      } else {
         form.C.value = My_Series.stringify( C, series.value, 'F' );
         set_deviation( form );
         My_Messages.put( "C = " + Ct_1_ln_2 + " / ( f * (R1 + 2 * R2) )" );
      };
   };
   return;
} // do_B5_cond

function do_B6_optimize ( form ) {
   My_Messages.reset();

   var f = calculate_period_and_frequency( form );
   if ( My_Messages.error() ) return;
   if ( !f ) {
      My_Messages.put_error( "f>0 oder t>0 wird benötigt" );
      return;
   };

   var RRC = calculate_resistors_and_capacitor( f, form, true );
   var R1  = RRC[0];
   var R2  = RRC[1];
   var C   = RRC[2];
   if ( !R1 || !R2 || !C ) {
      var tg = My_Series.real( form.tg.value, '', '' );
      if ( tg ) {
         My_Messages.put_error( "Für f = " + My_Series.stringify( f, 'Ing', 'Hz' ) +
                                " und tg=" + tg + " konnten R1, R2 und C nicht ermittelt werden." );
      } else {
         My_Messages.put_error( "Für f = " + My_Series.stringify( f, 'Ing', 'Hz' ) + " konnten R1, R2 und C nicht ermittelt werden." );
      };
      return
   };

   var series1    = form.R3.value;
   var series2    = form.R4.value;
   var seriesc    = form.R5.value;
   form.Rt1.value = My_Series.stringify( R1, series1, 'Ω' );
   form.Rt2.value = My_Series.stringify( R2, series2, 'Ω' );
   form.C.value   = My_Series.stringify(  C,  seriesc, 'F' );

   set_deviation( form );
   return;
}; // do_B6_optimize

function do_B7_t( form, series ) {
   My_Messages.reset();
   var f   = calculate_period_and_frequency( form );
   var Rt1 = My_Series.real( form.Rt1.value, 'Ω', 'R1' );
   var Rt2 = My_Series.real( form.Rt2.value, 'Ω', 'R2' );
   var C   = My_Series.real( form.C.value, 'F', 'C' );
   if ( My_Messages.error() ) return;
   if ( (!Rt1 || !Rt2 || !(f || C) ) ) {
      My_Messages.put_error( "Es werden R1 und R2 sowie f oder C benötigt." );
      suggest_RC( form );
   } else if ( Rt1 > C_max_R ) {
      My_Messages.put_error( "R1 = " + My_Series.stringify( Rt1, 'Ing', 'Ω' ) + " > " + My_Series.stringify( C_max_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else if ( Rt1 < C_min_R ) {
      My_Messages.put_error( "R1 = " + My_Series.stringify( Rt1, 'Ing', 'Ω' ) + " < " + My_Series.stringify( C_min_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else if ( Rt2 > C_max_R ) {
      My_Messages.put_error( "R2 = " + My_Series.stringify( Rt2, 'Ing', 'Ω' ) + " > " + My_Series.stringify( C_max_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else if ( Rt2 < C_min_R ) {
      My_Messages.put_error( "R2 = " + My_Series.stringify( Rt2, 'Ing', 'Ω' ) + " < " + My_Series.stringify( C_min_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else if ( f ) {
      var C = C_1_ln_2 / ( f * (Rt1 + 2 * Rt2) );
      if ( C < C_min_C ) {
         My_Messages.put_error( "C = " + My_Series.stringify( C, 'Ing', 'F' ) + " < " + My_Series.stringify( C_min_C, 'Ing', 'F' ) );
         suggest_RC( form );
      } else {
         var seriesc    = form.R5.value;
         form.C.value = My_Series.stringify( C, seriesc, 'F' );
         set_deviation( form );
         My_Messages.put( "C = " + Ct_1_ln_2 + " / ( f * (R1 + 2 * R2) )" );
      };
   } else {
      var seriesf    = form.R1.value;
      var seriest    = form.R2.value;
      form.f.value = My_Series.stringify( C_1_ln_2 / ( (Rt1 + 2 * Rt2) * C ), seriesf, 'Hz' );
      form.t.value = My_Series.stringify( ( (Rt1 + 2 * Rt2) * C ) / C_1_ln_2, seriest, 's' );
      set_deviation( form );
      My_Messages.put( "f = " + Ct_1_ln_2 + " / ( (R1 + 2 * R2) * C )" );
      My_Messages.put( "t = 1 / f = ( (R1 + 2 * R2) * C ) / " + Ct_1_ln_2 );
   };

   set_deviation( form );
   return;
} // do_B7_t

// Clear

function do_B0_clear ( form ) {
   My_Messages.reset();
   form.f.value = '';
   form.t.value = '';
   form.Rt1.value = '';
   form.Rt2.value = '';
   form.C.value = '';
   form.th.value = '';
   form.tl.value = '';
   form.tg.value = '';
   My_Deviation.reset();
   return;
}; // do_B0_clear

function do_B1_clear ( form ) {
   My_Messages.reset();
   form.f.value = '';
   My_Deviation.reset();
   return;
}; // do_B1_clear

function do_B2_clear ( form ) {
   My_Messages.reset();
   form.t.value = '';
   My_Deviation.reset();
   return;
}; // do_B2_clear

function do_B3_clear ( form ) {
   My_Messages.reset();
   form.Rt1.value = '';
   My_Deviation.reset();
   return;
}; // do_B3_clear

function do_B4_clear ( form ) {
   My_Messages.reset();
   form.Rt2.value = '';
   My_Deviation.reset();
   return;
}; // do_B4_clear

function do_B5_clear ( form ) {
   My_Messages.reset();
   form.C.value = '';
   My_Deviation.reset();
   return;
}; // do_B5_clear

function do_B6_clear ( form ) {
   My_Messages.reset();
   form.th.value = '';
   My_Deviation.reset();
   return;
}; // do_B6_clear

function do_B7_clear ( form ) {
   My_Messages.reset();
   form.tl.value = '';
   My_Deviation.reset();
   return;
}; // do_B7_clear

function do_B8_clear ( form ) {
   My_Messages.reset();
   form.tg.value = '';
   My_Deviation.reset();
   return;
}; // do_B8_clear

// Change series

function do_R1_exakt ( form ) {
   My_Messages.reset();
   var f        = My_Series.real( form.f.value, 'Hz', 'f' );
   form.f.value = My_Series.stringify( f, 'Exakt', 'Hz' );
   return;
}; // do_R1_exakt

function do_R1_ing ( form ) {
   My_Messages.reset();
   var f        = My_Series.real( form.f.value, 'Hz', 'f' );
   form.f.value = My_Series.stringify( f, 'Ing', 'Hz' );
   return;
}; // do_R1_ing

function do_R2_exakt ( form ) {
   My_Messages.reset();
   var t        = My_Series.real( form.t.value, 's', 't' );
   form.t.value = My_Series.stringify( t, 'Exakt', 's' );
   return;
}; // do_R2_exakt

function do_R2_ing ( form ) {
   My_Messages.reset();
   var t        = My_Series.real( form.t.value, 's', 't' );
   form.t.value = My_Series.stringify( t, 'Ing', 's' );
   return;
}; // do_R2_ing

function do_R3_exakt ( form ) {
   My_Messages.reset();
   var Rt1        = My_Series.real( form.Rt1.value, 'Ω', 'R1' );
   form.Rt1.value = My_Series.stringify( Rt1, 'Exakt', 'Ω' );
   set_deviation( form );
   return;
}; // do_R3_exakt

function do_R3_ing ( form ) {
   My_Messages.reset();
   var Rt1        = My_Series.real( form.Rt1.value, 'Ω', 'R1' );
   form.Rt1.value = My_Series.stringify( Rt1, 'Ing', 'Ω' );
   set_deviation( form );
   return;
}; // do_R3_ing

function do_R3_e6 ( form ) {
   My_Messages.reset();
   var Rt1        = My_Series.real( form.Rt1.value, 'Ω', 'R1' );
   form.Rt1.value = My_Series.stringify( Rt1, 'E6', 'Ω' );
   set_deviation( form );
   return;
}; // do_R3_e6

function do_R3_e12 ( form ) {
   My_Messages.reset();
   var Rt1        = My_Series.real( form.Rt1.value, 'Ω', 'R1' );
   form.Rt1.value = My_Series.stringify( Rt1, 'E12', 'Ω' );
   set_deviation( form );
   return;
}; // do_R3_e12

function do_R3_e24 ( form ) {
   My_Messages.reset();
   var Rt1        = My_Series.real( form.Rt1.value, 'Ω', 'R1' );
   form.Rt1.value = My_Series.stringify( Rt1, 'E24', 'Ω' );
   set_deviation( form );
   return;
}; // do_R3_e24

function do_R4_exakt ( form ) {
   My_Messages.reset();
   var Rt2        = My_Series.real( form.Rt2.value, 'Ω', 'R2' );
   form.Rt2.value = My_Series.stringify( Rt2, 'Exakt', 'Ω' );
   set_deviation( form );
   return;
}; // do_R4_exakt

function do_R4_ing ( form ) {
   My_Messages.reset();
   var Rt2        = My_Series.real( form.Rt2.value, 'Ω', 'R2' );
   form.Rt2.value = My_Series.stringify( Rt2, 'Ing', 'Ω' );
   set_deviation( form );
   return;
}; // do_R4_ing

function do_R4_e6 ( form ) {
   My_Messages.reset();
   var Rt2        = My_Series.real( form.Rt2.value, 'Ω', 'R2' );
   form.Rt2.value = My_Series.stringify( Rt2, 'E6', 'Ω' );
   set_deviation( form );
   return;
}; // do_R4_e6

function do_R4_e12 ( form ) {
   My_Messages.reset();
   var Rt2        = My_Series.real( form.Rt2.value, 'Ω', 'R2' );
   form.Rt2.value = My_Series.stringify( Rt2, 'E12', 'Ω' );
   set_deviation( form );
   return;
}; // do_R4_e12

function do_R4_e24 ( form ) {
   My_Messages.reset();
   var Rt2        = My_Series.real( form.Rt2.value, 'Ω', 'R2' );
   form.Rt2.value = My_Series.stringify( Rt2, 'E24', 'Ω' );
   set_deviation( form );
   return;
}; // do_R4_e24

function do_R5_exakt ( form ) {
   My_Messages.reset();
   var C        = My_Series.real( form.C.value, 'F', 'C' );
   form.C.value = My_Series.stringify( C, 'Exakt', 'F' );
   set_deviation( form );
   return;
}; // do_R5_exakt

function do_R5_ing ( form ) {
   My_Messages.reset();
   var C        = My_Series.real( form.C.value, 'F', 'C' );
   form.C.value = My_Series.stringify( C, 'Ing', 'F' );
   set_deviation( form );
   return;
}; // do_R5_ing

function do_R5_e3 ( form ) {
   My_Messages.reset();
   var C        = My_Series.real( form.C.value, 'F', 'C' );
   form.C.value = My_Series.stringify( C, 'E3', 'F' );
   set_deviation( form );
   return;
}; // do_R5_e3

function do_R5_e6 ( form ) {
   My_Messages.reset();
   var C        = My_Series.real( form.C.value, 'F', 'C' );
   form.C.value = My_Series.stringify( C, 'E6', 'F' );
   set_deviation( form );
   return;
}; // do_R5_e6

function do_R6_exakt ( form ) {
   My_Messages.reset();
   var th        = My_Series.real( form.th.value, 's', 't' );
   form.th.value = My_Series.stringify( th, 'Exakt', 's' );
   return;
}; // do_R6_exakt

function do_R6_ing ( form ) {
   My_Messages.reset();
   var th        = My_Series.real( form.th.value, 's', 't' );
   form.th.value = My_Series.stringify( th, 'Ing', 's' );
   return;
}; // do_R6_ing

function do_R7_exakt ( form ) {
   My_Messages.reset();
   var tl        = My_Series.real( form.tl.value, 's', 't' );
   form.tl.value = My_Series.stringify( tl, 'Exakt', 's' );
   return;
}; // do_R7_exakt

function do_R7_ing ( form ) {
   My_Messages.reset();
   var tl        = My_Series.real( form.tl.value, 's', 't' );
   form.tl.value = My_Series.stringify( tl, 'Ing', 's' );
   return;
}; // do_R7_ing

