
var My_Messages       = new TinyMessage( "Note" );
var My_Series         = new E_Series( My_Messages );
var My_Type           = new SimpleLabel( "TypMultivibrator" );
var My_Description    = new SimpleLabel( "LabelBeschreibung" );
var My_Deviation      = new SimpleLabel( "LabelAbweichung" );

var Monostab_Type     = {
                          HC123:     {
                                       image: "74HC123-monostabil.png",
                                       min_delay:    1e-9,
                                       min_capacity: 1e-10,
                                       min_resistor: 5000,
                                       max_resistor: 3000000,
                                       time_factor:  0.45,
                                       name:         "HC123",
                                       description:  "Trigger und Reset flankengesteuert, Schmitt Trigger an A und B, retriggerbar"
                                     },
                          HC221:     {
                                       image: "74HC221-monostabil.png",
                                       min_delay:    1e-9,
                                       min_capacity: 1e-10,
                                       min_resistor: 500,
                                       max_resistor: 3000000,
                                       time_factor:  0.7,
                                       name:         "HC221",
                                       description:  "Trigger und Reset flankengesteuert, Schmitt Trigger nur an B, nicht retriggerbar"
                                     },
                          HC423:     {
                                       image: "74HC423-monostabil.png",
                                       min_delay:    1e-9,
                                       min_capacity: 1e-10,
                                       min_resistor: 5000,
                                       max_resistor: 3000000,
                                       time_factor:  0.45,
                                       name:         "HC423",
                                       description:  "Trigger flankengesteuert, Reset zustandsgesteuert, Schmitt Trigger an A und B, retriggerbar"
                                     },
                          HC4538:    {
                                       image: "74HC4538-monostabil.png",
                                       min_delay:    1e-9,
                                       min_capacity: 1e-10,
                                       min_resistor: 5000,
                                       max_resistor: 3000000,
                                       time_factor:  0.7,
                                       name:         "HC4538",
                                       description:  "Trigger flankengesteuert, Reset zustandsgesteuert, Schmitt Trigger an A und B, retriggerbar"
                                     },
                          Timer555:  {
                                       image: "Ne555-monostabil.png",
                                       min_delay:    1e-9,
                                       min_capacity: 1e-10,
                                       min_resistor: 1000,
                                       max_resistor: 3000000,
                                       time_factor:  1.1,
                                       name:         "Timer 555",
                                       description:  "Trigger und Reset zustandsgesteuert, Trigger negativ mit U/3 Schwelle, nicht retriggerbar"
                                     },
                          Timer555C: {
                                       image: "Ne555-monostabil-C.png",
                                       min_delay:    1e-9,
                                       min_capacity: 1e-10,
                                       min_resistor: 1000,
                                       max_resistor: 3000000,
                                       time_factor:  1.1,
                                       name:         "Timer 555 mit C",
                                       description:  "Trigger flankengesteuert, Reset zustandsgesteuert, nicht retriggerbar"
                                     }
                        };

var Current_Type = "HC123";

function Mono_init_me () {
   document.form1.R1[0].checked=true;
   Current_Type = document.form1.R1[0].value;
   My_Description.put( Monostab_Type[Current_Type]["description"] );
   My_Type.put( Monostab_Type[Current_Type]["name"] );
   My_Deviation.reset();
   My_Messages.reset();
   document.form1.R.value = '';
   document.form1.C.value = '';
   return;
}; // Mono_init_me

function calculate_resistor_and_capacitor ( delay ) {

   if ( !delay ) return;

   var C            = Monostab_Type[Current_Type]["min_capacity"];
   var time_factor  = Monostab_Type[Current_Type]["time_factor"];
   var min_resistor = Monostab_Type[Current_Type]["min_resistor"];
   var max_resistor = Monostab_Type[Current_Type]["max_resistor"];

   var R            = delay / ( time_factor * C );
   var limit        = 0;
   while ( ! ( ( R >= min_resistor) && ( R <= max_resistor ) ) ) {
      C = C * 10;
      R = delay / ( time_factor * C );
      if ( limit > 10000 ) return;
      limit++;
   };

   return [ R, C ];

}; // calculate_resistor_and_capacitor

function suggest_RC ( form, series_value ) {
   var t  = My_Series.real( form.t.value, 's', 't' );
   var RC = calculate_resistor_and_capacitor( t );
   var Rx = RC[0];
   var Cx = RC[1];
   My_Messages.put( "C=" + My_Series.stringify( Cx, 'Ing', 'F' ) +
                    " und R=" + My_Series.stringify( Rx, series_value, 'Ω' ) + " wären möglich."
                  );
   return;
}; // suggest_RC

function set_deviation ( form ) {
   var t             = My_Series.real( form.t.value, 's', 't' );
   var R             = My_Series.real( form.R.value, 'Ω', 'R' );
   var C             = My_Series.real( form.C.value, 'F', 'C' );
   if ( My_Messages.error() || !t || !R || !C ) {
      My_Deviation.reset();
      return;
   };
   var time_factor   = Monostab_Type[Current_Type]["time_factor"];
   var T_value       = time_factor * C * R;
   var deviation     = 100 * ( t - T_value ) / t;
   if ( Math.abs(deviation) < 0.1 ) deviation = 0;
   My_Deviation.put( "Der berechnete Wert t weicht um " + deviation.toFixed( 2 ) + "% ab." );
   return;
}; // set_deviation

// Set type of monostable

function do_R1_hc123( form, type ) {
   My_Messages.reset();
   Current_Type = type.value;
   document.image1.src = Monostab_Type[Current_Type]["image"];
   My_Description.put( Monostab_Type[Current_Type]["description"] );
   My_Type.put( Monostab_Type[Current_Type]["name"] );
   if ( form.t.value && form.R.value && form.C.value ) {
      form.R.value = '';
      form.C.value = '';
   };
   My_Deviation.reset();
   return;
}; // do_R1_hc123

function do_R1_hc221( form, type ) {
   do_R1_hc123( form, type );
   return;
}; // do_R1_hc221

function do_R1_hc423( form, type ) {
   do_R1_hc123( form, type );
   return;
}; // do_R1_hc423

function do_R1_hc4538( form, type ) {
   do_R1_hc123( form, type );
   return;
}; // do_R1_hc4538

function do_R1_ne555( form, type ) {
   do_R1_hc123( form, type );
   return;
}; // do_R1_ne555

function do_R1_nec555( form, type ) {
   do_R1_hc123( form, type );
   return;
}; // do_R1_ne555

// Do calculations

function do_B2_haltezeit( form, series ) {
   My_Messages.reset();
   var R = My_Series.real( form.R.value, 'Ω', 'R' );
   var C = My_Series.real( form.C.value, 'F', 'C' );
   var min_capacitor = Monostab_Type[Current_Type]["min_capacity"];
   var time_factor   = Monostab_Type[Current_Type]["time_factor"];
   var min_resistor  = Monostab_Type[Current_Type]["min_resistor"];
   var max_resistor  = Monostab_Type[Current_Type]["max_resistor"];
   if ( My_Messages.error() ) return;
   if ( (!R && !C) ) {
      My_Messages.put_error( "Es werden R>0 und C>0 benötigt." );
   } else if ( R > max_resistor ) {
      My_Messages.put_error( "R = " + My_Series.stringify( R, 'Ing', 'Ω' ) + " > " + My_Series.stringify( max_resistor, 'Ing', 'Ω' ) );
      return;
   } else if ( R < min_resistor ) {
      My_Messages.put_error( "R = " + My_Series.stringify( R, 'Ing', 'Ω' ) + " < " + My_Series.stringify( min_resistor, 'Ing', 'Ω' ) );
      return;
   } else if ( C < min_capacitor ) {
      My_Messages.put_error( "C = " + My_Series.stringify( C, 'Ing', 'F' ) + " < " + My_Series.stringify( min_capacitor, 'Ing', 'F' ) );
      return;
   } else {
      form.t.value = My_Series.stringify( time_factor * C * R, series.value, 's' );
      set_deviation( form );
   };
   return;
} // do_B2_haltezeit

function do_B2_clear ( form ) {
   My_Messages.reset();
   form.t.value = '';
   My_Deviation.reset();
   return;
}; // do_B2_clear

function do_B3_ohm( form, series ) {
   My_Messages.reset();
   var C             = My_Series.real( form.C.value, 'F', 'C' );
   var t             = My_Series.real( form.t.value, 's', 't' );
   var min_capacitor =  Monostab_Type[Current_Type]["min_capacity"];
   var time_factor   = Monostab_Type[Current_Type]["time_factor"];
   var min_resistor  = Monostab_Type[Current_Type]["min_resistor"];
   var max_resistor  = Monostab_Type[Current_Type]["max_resistor"];
   var min_C_string  = My_Series.stringify( min_capacitor, 'Ing', 'F' );
   if ( My_Messages.error() ) return;
   if ( (!C && !t ) ) {
      My_Messages.put_error( "Es werden C>" + min_C_string + " und t>0 benötigt." );
      suggest_RC( form, series.value );
   } if ( C == '' ) {
      My_Messages.put_error( "Es wird C>"+ min_C_string + " benötigt." );
      suggest_RC( form, series.value );
   } else if ( C < min_capacitor ) {
      My_Messages.put_error( "C = " + My_Series.stringify( C, 'Ing', 'F' ) + " < " + min_C_string );
      suggest_RC( form, series.value );
   } else {
      var R = t / ( time_factor * C );
      if ( R > max_resistor ) {
         My_Messages.put_error( "R = " + My_Series.stringify( R, 'Ing', 'Ω' ) + " > " + My_Series.stringify( max_resistor, 'Ing', 'Ω' ) );
         suggest_RC( form, series.value );
      } else if ( R < min_resistor ) {
         My_Messages.put_error( "R = " + My_Series.stringify( R, 'Ing', 'Ω' ) + " < " + My_Series.stringify( min_resistor, 'Ing', 'Ω' ) );
         suggest_RC( form, series.value );
      } else {
         form.R.value = My_Series.stringify( R, series.value, 'Ω' );
         set_deviation( form );
      };
   };
   return;
} // do_B3_ohm

function do_B3_optimize ( form, series ) {
   My_Messages.reset();
   var t  = My_Series.real( form.t.value, 's', 't' );
   if ( My_Messages.error() ) return;
   if ( (!t ) ) {
      My_Messages.put_error( "Es wird t>0 benötigt." );
      return;
   };
   var RC = calculate_resistor_and_capacitor( t );
   var R  = RC[0];
   var C  = RC[1];
   form.R.value = My_Series.stringify( R, series.value, 'Ω' );
   form.C.value = My_Series.stringify( C, 'Ing', 'F' );
   set_deviation( form );
   return;
}; // do_B3_optimize

function do_B3_clear ( form ) {
   My_Messages.reset();
   form.R.value = '';
   My_Deviation.reset();
   return;
}; // do_B3_clear

function do_B4_cond( form, series ) {
   My_Messages.reset();
   var R             = My_Series.real( form.R.value, 'Ω', 'R' );
   var t             = My_Series.real( form.t.value, 's', 't' );
   var min_capacitor = Monostab_Type[Current_Type]["min_capacity"];
   var time_factor   = Monostab_Type[Current_Type]["time_factor"];
   var min_resistor  = Monostab_Type[Current_Type]["min_resistor"];
   var max_resistor  = Monostab_Type[Current_Type]["max_resistor"];
   var min_R_string  = My_Series.stringify( min_resistor, 'Ing', 'Ω' );
   var max_R_string  = My_Series.stringify( max_resistor, 'Ing', 'Ω' );
   if ( My_Messages.error() ) return;
   if ( (!R && !t ) ) {
      My_Messages.put_error( "Es werden R>" + min_R_string + "<" + max_R_string + " und t>0 benötigt." );
      suggest_RC( form, series.value );
   } if ( R == '' ) {
      My_Messages.put_error( "Es wird R>" + min_R_string + "<" + max_R_string  + " benötigt." );
      suggest_RC( form, series.value );
   } else if ( R < min_resistor ) {
      My_Messages.put_error( "R = " + My_Series.stringify( R, 'Ing', 'Ω' ) + " < " + min_R_string );
      suggest_RC( form, series.value );
   } else if ( R > max_resistor ) {
      My_Messages.put_error( "R = " + My_Series.stringify( R, 'Ing', 'Ω' ) + " > " + max_R_string );
      suggest_RC( form, series.value );
   } else {
      var C = t / ( time_factor * R );
      if ( C < min_capacitor ) {
         My_Messages.put_error( "R = " + My_Series.stringify( C, 'Ing', 'F' ) + " < " + My_Series.stringify( min_capacitor, 'Ing', 'F' ) );
         suggest_RC( form, series.value );
      } else {
         form.C.value = My_Series.stringify( C, series.value, 'F' );
         set_deviation( form );
      };
   };
   return;
} // do_B4_cond

function do_B4_clear ( form ) {
   My_Messages.reset();
   form.C.value = '';
   My_Deviation.reset();
   return;
}; // do_B4_clear

// Change series

function do_R2_exakt ( form ) {
   My_Messages.reset();
   var t        = My_Series.real( form.t.value, 's', 't' );
   form.t.value = My_Series.stringify( t, 'Exakt', 's' );
   return;
}; // do_R2_exakt

function do_R2_ing ( form ) {
   My_Messages.reset();
   var t        = My_Series.real( form.t.value, 's', 't' );
   form.t.value = My_Series.stringify( t, 'Ing', 's' );
   return;
}; // do_R2_ing

function do_R3_exakt ( form ) {
   My_Messages.reset();
   var R        = My_Series.real( form.R.value, 'Ω', 'R' );
   form.R.value = My_Series.stringify( R, 'Exakt', 'Ω' );
   set_deviation( form );
   return;
}; // do_R3_exakt

function do_R3_ing ( form ) {
   My_Messages.reset();
   var R        = My_Series.real( form.R.value, 'Ω', 'R' );
   form.R.value = My_Series.stringify( R, 'Ing', 'Ω' );
   set_deviation( form );
   return;
}; // do_R3_ing

function do_R3_e6 ( form ) {
   My_Messages.reset();
   var R        = My_Series.real( form.R.value, 'Ω', 'R' );
   form.R.value = My_Series.stringify( R, 'E6', 'Ω' );
   set_deviation( form );
   return;
}; // do_R3_e6

function do_R3_e12 ( form ) {
   My_Messages.reset();
   var R        = My_Series.real( form.R.value, 'Ω', 'R' );
   form.R.value = My_Series.stringify( R, 'E12', 'Ω' );
   set_deviation( form );
   return;
}; // do_R3_e12

function do_R3_e24 ( form ) {
   My_Messages.reset();
   var R        = My_Series.real( form.R.value, 'Ω', 'R' );
   form.R.value = My_Series.stringify( R, 'E24', 'Ω' );
   set_deviation( form );
   return;
}; // do_R3_e24

function do_R4_exakt ( form ) {
   My_Messages.reset();
   var R        = My_Series.real( form.R.value, 'Ω', 'R' );
   form.R.value = My_Series.stringify( R, 'Exakt', 'Ω' );
   set_deviation( form );
   return;
}; // do_R4_exakt

function do_R4_ing ( form ) {
   My_Messages.reset();
   var C        = My_Series.real( form.C.value, 'F', 'C' );
   form.C.value = My_Series.stringify( C, 'Ing', 'F' );
   set_deviation( form );
   return;
}; // do_R4_ing

function do_R4_e3 ( form ) {
   My_Messages.reset();
   var C        = My_Series.real( form.C.value, 'F', 'C' );
   form.C.value = My_Series.stringify( C, 'E3', 'F' );
   set_deviation( form );
   return;
}; // do_R4_e3

function do_R4_e6 ( form ) {
   My_Messages.reset();
   var C        = My_Series.real( form.C.value, 'F', 'C' );
   form.C.value = My_Series.stringify( C, 'E6', 'F' );
   set_deviation( form );
   return;
}; // do_R4_e6

