
var My_Messages       = new TinyMessage( "Note" );
var My_Series         = new E_Series( My_Messages );
var My_Deviation      = new SimpleLabel( "LabelAbweichung" );

var C_max_frequency = 500e3;
var C_min_C         = 100e-12;
var C_max_C         = 10e-3;
var C_min_R         = 4.7e3;
var C_max_R         = 4.7e6;

var C_K             = 1.1;

function Oszillator_74HC14_init_me () {
   My_Deviation.reset();
   My_Messages.reset();
   document.form1.R.value = '';
   document.form1.C.value   = '';
   return;
}; // Oszillator_74HC14_init_me

function calculate_period_and_frequency ( form ) {

   var f        = My_Series.real( form.f.value,  'Hz', 'f' );
   var t        = My_Series.real( form.t.value,  's',  't' );
   var seriesf  = form.R1.value;
   var seriest  = form.R2.value;

   if ( !t && !f ) return '';
   if ( f ) {
      if ( f > C_max_frequency ) {
         My_Messages.put_error( "f=" + My_Series.stringify( f, 'Ing', 'Hz' ) + ">" +
                                       My_Series.stringify( C_max_frequency, 'Ing', 'Hz' ) + " ist zu groß."
                              );
         return '';
      };
      t = 1 / f;
      form.t.value = My_Series.stringify( t, seriest, 's' );
      return f;
   } else if ( 1/t > C_max_frequency ) {
      My_Messages.put_error( "t=" + My_Series.stringify( t, 'Ing', 's' ) + "<" +
                                    My_Series.stringify( 1/C_max_frequency, 'Ing', 's' ) + " ist zu kein."
                           );
      return '';
   } else {
      form.f.value = My_Series.stringify( 1/t, seriesf, 'Hz' );
      return 1 / t;
   };
}; // calculate_period_and_frequency

function calculate_resistor_and_capacitor ( f, form, info ) {

   if ( !f )                  return [ '', '', '' ];
   if ( f < 0 )               return [ '', '', '' ];
   if ( f > C_max_frequency ) return [ '', '', '' ];

   var C       = C_min_C;
   var min_R   = C_min_R;
   var max_R   = C_max_R;
   // first try
   var R       = C_K / ( f  * C );
   var limit  = 0;
   while ( ! ( (R >= min_R) && (R <= max_R) ) ) {
      C = C * 10;
      R = C_K / ( f  * C );
      if ( limit > 10000 ) return [ '', '' ];
      limit++;
   };

   return [ R, C ];

}; // calculate_resistor_and_capacitor

function suggest_RC ( form ) {
   var f   = calculate_period_and_frequency( form );
   var RC = calculate_resistor_and_capacitor( f, form, false );
   var R  = RC[0];
   var C  = RC[1];
   if ( !f || !R || !C ) return;
   My_Messages.put( "C="       + My_Series.stringify( C, 'Ing', 'F' ) +
                    " und R =" + My_Series.stringify( R, 'Ing', 'Ω' ) + " wären möglich."
                  );
   return;
}; // suggest_RC

function set_deviation ( form ) {
   var f = calculate_period_and_frequency( form );
   var t = My_Series.real( form.t.value, 's', 't' );
   var R = My_Series.real( form.R.value, 'Ω', 'R' );
   var C = My_Series.real( form.C.value, 'F', 'C' );
   if ( My_Messages.error() || !f || !R || !C ) {
      My_Deviation.reset();
      return;
   };

   var f_value   = C_K / ( R * C );
   var deviation = 100 * ( f - f_value ) / f;
   if ( Math.abs(deviation) < 0.1 ) deviation = 0;
   My_Deviation.put( "Der berechnete Wert f weicht um " + deviation.toFixed( 2 ) + "% ab." );
   return;
}; // set_deviation

// Do calculations

function do_B1_frequenz( form, series ) {
   My_Messages.reset();
   var R = My_Series.real( form.R.value, 'Ω', 'R' );
   var C = My_Series.real( form.C.value, 'F', 'C' );
   if ( My_Messages.error() ) return;
   if ( (!R || !C) ) {
      My_Messages.put_error( "Es werden R>0 und C>0 benötigt." );
      suggest_RC( form );
   } else if ( R > C_max_R ) {
      My_Messages.put_error( "R = " + My_Series.stringify( R, 'Ing', 'Ω' ) + " > " + My_Series.stringify( C_max_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else if ( R < C_min_R ) {
      My_Messages.put_error( "R = " + My_Series.stringify( R, 'Ing', 'Ω' ) + " < " + My_Series.stringify( C_min_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else if ( C < C_min_C ) {
      My_Messages.put_error( "C = " + My_Series.stringify( C, 'Ing', 'F' ) + " < " + My_Series.stringify( C_min_C, 'Ing', 'F' ) );
      suggest_RC( form );
   } else {
      form.f.value = My_Series.stringify( C_K / ( R * C ), series.value, 'Hz' );
      form.t.value = My_Series.stringify( ( R * C ) / C_K, series.value, 's' );
      set_deviation( form );
      My_Messages.put( "f = " + C_K + " / ( R * C )" );
      My_Messages.put( "t = 1 / f = ( R * C ) / " + C_K );
   };
   return;
} // do_B1_frequenz

function do_B2_periode( form, series ) {
   do_B1_frequenz( form, series );
   return;
} // do_B2_periode

function do_B3_ohm( form, series ) {
   My_Messages.reset();
   var f = calculate_period_and_frequency( form );
   var C = My_Series.real( form.C.value, 'F', 'C' );
   if ( My_Messages.error() ) return;
   if ( ( !C || !f ) ) {
      My_Messages.put_error( "Es werden C und f benötigt." );
      suggest_RC( form );
   } else if ( C == '' ) {
      My_Messages.put_error( "Es wird C>"+ My_Series.stringify( C_min_C, 'Ing', 'F' ) + " benötigt." );
      suggest_RC( form );
   } else if ( C < C_min_C ) {
      My_Messages.put_error( "C = " + My_Series.stringify( C, 'Ing', 'F' ) + " < " + My_Series.stringify( C_min_C, 'Ing', 'F' ) );
      suggest_RC( form );
   } else {
      var R = (C_K / ( f * C ));
      if ( R > C_max_R ) {
         My_Messages.put_error( "R = " + My_Series.stringify( R, 'Ing', 'Ω' ) + " > " + My_Series.stringify( C_max_R, 'Ing', 'Ω' ) );
         suggest_RC( form );
      } else if ( R < C_min_R ) {
         My_Messages.put_error( "R = " + My_Series.stringify( R, 'Ing', 'Ω' ) + " < " + My_Series.stringify( C_min_R, 'Ing', 'Ω' ) );
         suggest_RC( form );
      } else {
         form.R.value = My_Series.stringify( R, series.value, 'Ω' );
         set_deviation( form );
         My_Messages.put( "R = " + C_K + " / (f * C)" );
      };
   };
   return;
} // do_B3_ohm

function do_B4_cond( form, series ) {
   My_Messages.reset();
   var f = calculate_period_and_frequency( form );
   var R = My_Series.real( form.R.value, 'Ω', 'R' );
   if ( My_Messages.error() ) return;
   if ( !R || !f ) {
      My_Messages.put_error( "Es werden R und f benötigt." );
      suggest_RC( form );
   } else if ( R > C_max_R ) {
      My_Messages.put_error( "R = " + My_Series.stringify( R, 'Ing', 'Ω' ) + " > " + My_Series.stringify( C_max_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else if ( R < C_min_R ) {
      My_Messages.put_error( "R = " + My_Series.stringify( R, 'Ing', 'Ω' ) + " < " + My_Series.stringify( C_min_R, 'Ing', 'Ω' ) );
      suggest_RC( form );
   } else {
      var C = C_K / ( f * R );
      if ( C < C_min_C ) {
         My_Messages.put_error( "C = " + My_Series.stringify( C, 'Ing', 'F' ) + " < " + My_Series.stringify( C_min_C, 'Ing', 'F' ) );
         suggest_RC( form );
      } else {
         form.C.value = My_Series.stringify( C, series.value, 'F' );
         set_deviation( form );
         My_Messages.put( "C = " + C_K + " / ( f * R )" );
      };
   };
   return;
} // do_B4_cond

function do_B0_optimize ( form ) {
   My_Messages.reset();

   var f = calculate_period_and_frequency( form );
   if ( My_Messages.error() ) return;
   if ( !f ) {
      My_Messages.put_error( "f>0 oder t>0 wird benötigt" );
      return;
   };

   var RC = calculate_resistor_and_capacitor( f, form, true );
   var R  = RC[0];
   var C  = RC[1];
   if ( !R || !C ) {
      My_Messages.put_error( "Für f = " + My_Series.stringify( f, 'Ing', 'Hz' ) + " konnten R und C nicht ermittelt werden." );
      return
   };

   var seriesR  = form.R3.value;
   var seriesc  = form.R4.value;
   form.R.value = My_Series.stringify( R, seriesR, 'Ω' );
   form.C.value = My_Series.stringify(  C,  seriesc, 'F' );

   set_deviation( form );
   return;
}; // do_B0_optimize

// Clear

function do_B0_clear ( form ) {
   My_Messages.reset();
   form.f.value = '';
   form.t.value = '';
   form.R.value = '';
   form.C.value = '';
   My_Deviation.reset();
   return;
}; // do_B0_clear

function do_B1_clear ( form ) {
   My_Messages.reset();
   form.f.value = '';
   My_Deviation.reset();
   return;
}; // do_B1_clear

function do_B2_clear ( form ) {
   My_Messages.reset();
   form.t.value = '';
   My_Deviation.reset();
   return;
}; // do_B2_clear

function do_B3_clear ( form ) {
   My_Messages.reset();
   form.R.value = '';
   My_Deviation.reset();
   return;
}; // do_B3_clear

function do_B4_clear ( form ) {
   My_Messages.reset();
   form.C.value = '';
   My_Deviation.reset();
   return;
}; // do_B4_clear

// Change series

function do_R1_exakt ( form ) {
   My_Messages.reset();
   var f        = My_Series.real( form.f.value, 'Hz', 'f' );
   form.f.value = My_Series.stringify( f, 'Exakt', 'Hz' );
   return;
}; // do_R1_exakt

function do_R1_ing ( form ) {
   My_Messages.reset();
   var f        = My_Series.real( form.f.value, 'Hz', 'f' );
   form.f.value = My_Series.stringify( f, 'Ing', 'Hz' );
   return;
}; // do_R1_ing

function do_R2_exakt ( form ) {
   My_Messages.reset();
   var t        = My_Series.real( form.t.value, 's', 't' );
   form.t.value = My_Series.stringify( t, 'Exakt', 's' );
   return;
}; // do_R2_exakt

function do_R2_ing ( form ) {
   My_Messages.reset();
   var t        = My_Series.real( form.t.value, 's', 't' );
   form.t.value = My_Series.stringify( t, 'Ing', 's' );
   return;
}; // do_R2_ing

function do_R3_exakt ( form ) {
   My_Messages.reset();
   var R        = My_Series.real( form.R.value, 'Ω', 'R' );
   form.R.value = My_Series.stringify( R, 'Exakt', 'Ω' );
   set_deviation( form );
   return;
}; // do_R3_exakt

function do_R3_ing ( form ) {
   My_Messages.reset();
   var R        = My_Series.real( form.R.value, 'Ω', 'R' );
   form.R.value = My_Series.stringify( R, 'Ing', 'Ω' );
   set_deviation( form );
   return;
}; // do_R3_ing

function do_R3_e6 ( form ) {
   My_Messages.reset();
   var R        = My_Series.real( form.R.value, 'Ω', 'R' );
   form.R.value = My_Series.stringify( R, 'E6', 'Ω' );
   set_deviation( form );
   return;
}; // do_R3_e6

function do_R3_e12 ( form ) {
   My_Messages.reset();
   var R        = My_Series.real( form.R.value, 'Ω', 'R' );
   form.R.value = My_Series.stringify( R, 'E12', 'Ω' );
   set_deviation( form );
   return;
}; // do_R3_e12

function do_R3_e24 ( form ) {
   My_Messages.reset();
   var R        = My_Series.real( form.R.value, 'Ω', 'R' );
   form.R.value = My_Series.stringify( R, 'E24', 'Ω' );
   set_deviation( form );
   return;
}; // do_R3_e24

function do_R4_exakt ( form ) {
   My_Messages.reset();
   var C        = My_Series.real( form.C.value, 'F', 'C' );
   form.C.value = My_Series.stringify( C, 'Exakt', 'F' );
   set_deviation( form );
   return;
}; // do_R4_exakt

function do_R4_ing ( form ) {
   My_Messages.reset();
   var C        = My_Series.real( form.C.value, 'F', 'C' );
   form.C.value = My_Series.stringify( C, 'Ing', 'F' );
   set_deviation( form );
   return;
}; // do_R4_ing

function do_R4_e3 ( form ) {
   My_Messages.reset();
   var C        = My_Series.real( form.C.value, 'F', 'C' );
   form.C.value = My_Series.stringify( C, 'E3', 'F' );
   set_deviation( form );
   return;
}; // do_R4_e3

function do_R4_e6 ( form ) {
   My_Messages.reset();
   var C        = My_Series.real( form.C.value, 'F', 'C' );
   form.C.value = My_Series.stringify( C, 'E6', 'F' );
   set_deviation( form );
   return;
}; // do_R4_e6

function do_R4_exakt ( form ) {
   My_Messages.reset();
   var C        = My_Series.real( form.C.value, 'F', 'C' );
   form.C.value = My_Series.stringify( C, 'Exakt', 'F' );
   return;
}; // do_R4_exakt

function do_R4_ing ( form ) {
   My_Messages.reset();
   var C        = My_Series.real( form.C.value, 'F', 'C' );
   form.C.value = My_Series.stringify( C, 'Ing', 'F' );
   return;
}; // do_R4_ing

